Struct regalloc2::MachineEnv
source · [−]pub struct MachineEnv {
pub preferred_regs_by_class: [Vec<PReg>; 2],
pub non_preferred_regs_by_class: [Vec<PReg>; 2],
pub scratch_by_class: [PReg; 2],
pub fixed_stack_slots: Vec<PReg>,
}Expand description
A machine envrionment tells the register allocator which registers are available to allocate and what register may be used as a scratch register for each class, and some other miscellaneous info as well.
Fields
preferred_regs_by_class: [Vec<PReg>; 2]Preferred physical registers for each class. These are the registers that will be allocated first, if free.
non_preferred_regs_by_class: [Vec<PReg>; 2]Non-preferred physical registers for each class. These are the registers that will be allocated if a preferred register is not available; using one of these is considered suboptimal, but still better than spilling.
scratch_by_class: [PReg; 2]One scratch register per class. This is needed to perform moves between registers when cyclic move patterns occur. The register should not be placed in either the preferred or non-preferred list (i.e., it is not otherwise allocatable).
Note that the register allocator will freely use this register between instructions, but within the machine code generated by a single (regalloc-level) instruction, the client is free to use the scratch register. E.g., if one “instruction” causes the emission of two machine-code instructions, this lowering can use the scratch register between them.
fixed_stack_slots: Vec<PReg>Some PRegs can be designated as locations on the stack rather than
actual registers. These can be used to tell the register allocator about
pre-defined stack slots used for function arguments and return values.
PRegs in this list cannot be used as a scratch register or as an
allocatable regsiter.